#pragma once
#include <map>
#include <tec/TecFrame.h>
#include <list>

enum class ValueType { Int, Float };
enum class Direction { Read, ReadWrite };
struct ParameterDescriptor {
  /// Base name of the process varable (aka DOOCS property), without ".RD", ".RW" suffix etc.
  std::string pvName;

  /// TecParameter identifier for the hardware protocol
  TecFrame::TecParameter value;

  /// Value type
  ValueType valueType;

  /// Direction
  Direction direction;

  /// Engineering unit
  std::string unit;

  /// Parameter desccription
  std::string description;

  /// Base name of the process variable for the read-group command
  std::string readGroup;

  /// Base name of the process variable for the write-group command, empty for Direction::Read
  std::string writeGroup;

  /// Base name of the process variable for the write-to-flash-group command, empty for Direction::Read
  std::string writeToFlashGroup;

  /// Base name of the proess variable for the set-from-readback-group command, empty for Direction::Read
  std::string setFromReadbackGroup;
};

static std::list<ParameterDescriptor> descriptors{

    /*
    {"mon_ch1_sinkTemp", TecFrame::TecParameter::MonSinkTemperature, ValueType::Float, Direction::Read, "unit",
    "description", "readCh1Group1", "", "", ""},

    {"mon_serialNumber", TecFrame::TecParameter::MonSerialNumber, ValueType::Int, Direction::Read, "unit",
    "description", "readGroup1", "", "", ""},

    {"ctrl_ch1_fanControlEnable", TecFrame::TecParameter::ExpFanControlEnable, ValueType::Int, Direction::ReadWrite,
    "unit", "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},

    {"ctrl_ch1_targetObjectTemp", TecFrame::TecParameter::TemTargetObjectTemp, ValueType::Float, Direction::ReadWrite,
    "unit", "description", "readCh1WriteGroup1", "writeCh1Group1", "writeToFlashCh1Group1", "setRbCh1Group1"}

    */
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // start ticket #6531
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // READ ONLY parameters – device specific
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    // read-only group 1 generic device parameters
    {"mon_deviceType", TecFrame::TecParameter::DeviceType, ValueType::Int, Direction::Read, "unit", "description",
        "readGroup1", "", "", ""},
    {"mon_hardwareVersion", TecFrame::TecParameter::MonHardwareVersion, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup1", "", "", ""},
    {"mon_firmwareVersion", TecFrame::TecParameter::MonFirmwareVersion, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup1", "", "", ""},
    {"mon_firmwareBuildNumber", TecFrame::TecParameter::MonFirmwareBuildNumber, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup1", "", "", ""},
    {"mon_serialNumber", TecFrame::TecParameter::MonSerialNumber, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup1", "", "", ""},
    // read-only group 2 generic device parameters
    {"mon_driverStatus", TecFrame::TecParameter::MonDriverStatus, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup2", "", "", ""},
    {"mon_parameterSystemFlashStatus", TecFrame::TecParameter::MonParameterSystemFlashStatus, ValueType::Int,
        Direction::Read, "unit", "description", "readGroup2", "", "", ""},

    // read-only group 3 generic device parameters
    {"mon_driverInputVoltage", TecFrame::TecParameter::MonDriverInputVoltage, ValueType::Float, Direction::Read, "unit",
        "description", "readGroup3", "", "", ""},
    {"mon_mediumInternalSupply", TecFrame::TecParameter::MonMedVInternalSupply, ValueType::Float, Direction::Read,
        "unit", "description", "readGroup3", "", "", ""},
    {"mon_33VInternalSupply", TecFrame::TecParameter::Mon33VInternalSupply, ValueType::Float, Direction::Read, "unit",
        "description", "readGroup3", "", "", ""},
    {"mon_basePlateTemp", TecFrame::TecParameter::MonBasePlateTemperature, ValueType::Float, Direction::Read, "unit",
        "description", "readGroup3", "", "", ""},

    // read-only group 4 generic device parameters
    {"mon_errorNumber", TecFrame::TecParameter::MonErrorNumber, ValueType::Int, Direction::Read, "unit", "description",
        "readGroup4", "", "", ""},
    {"mon_errorInstance", TecFrame::TecParameter::MonErrorInstance, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup4", "", "", ""},
    {"mon_errorParameter", TecFrame::TecParameter::MonErrorParameter, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup4", "", "", ""},

    // read-only group 5 generic device parameters
    {"mon_temperatureIsStable", TecFrame::TecParameter::MonTemperatureIsStable, ValueType::Int, Direction::Read, "unit",
        "description", "readGroup5", "", "", ""},

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // READ ONLY parameters – Channel specific
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

    // read-only group 1 for channel 1
    {"mon_ch1_objectTemp", TecFrame::TecParameter::MonObjectTemperature, ValueType::Float, Direction::Read, "C",
        "description", "readCh1Group1", "", "", ""},
    {"mon_ch1_sinkTemp", TecFrame::TecParameter::MonSinkTemperature, ValueType::Float, Direction::Read, "C",
        "description", "readCh1Group1", "", "", ""},
    // read-only group 2 for channel 1
    {"mon_ch1_targetObjectTemp", TecFrame::TecParameter::MonTargetObjectTemperature, ValueType::Float, Direction::Read,
        "C", "description", "readCh1Group2", "", "", ""},
    {"mon_ch1_rampNominalObjectTemp", TecFrame::TecParameter::MonRampNominalObjectTemperature, ValueType::Float,
        Direction::Read, "C", "description", "readCh1Group2", "", "", ""},
    {"mon_ch1_thermalPowerModelCurrent", TecFrame::TecParameter::MonThermalPowerModelCurrent, ValueType::Float,
        Direction::Read, "A", "description", "readCh1Group2", "", "", ""},
    // read-only group 3 for channel 1
    {"mon_ch1_actualOutputCurrent", TecFrame::TecParameter::MonActualOutputCurrent, ValueType::Float, Direction::Read,
        "A", "description", "readCh1Group3", "", "", ""},
    {"mon_ch1_actualOutputVoltage", TecFrame::TecParameter::MonActualOutputVoltage, ValueType::Float, Direction::Read,
        "V", "description", "readCh1Group3", "", "", ""},
    // read-only group 4
    {"mon_ch1_pidLowerLimitation", TecFrame::TecParameter::MonPIDLowerLimitation, ValueType::Float, Direction::Read, "",
        "description", "readCh1Group4", "", "", ""},
    {"mon_ch1_pidUpperLimitation", TecFrame::TecParameter::MonPIDUpperLimitation, ValueType::Float, Direction::Read, "",
        "description", "readCh1Group4", "", "", ""},
    {"mon_ch1_pidControlVariable", TecFrame::TecParameter::MonPIDControlVariable, ValueType::Float, Direction::Read, "",
        "description", "readCh1Group4", "", "", ""},
    // read-only group 5 for channel 1
    {"mon_ch1_objectSensorResistance", TecFrame::TecParameter::MonObjectSensorResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group5", "", "", ""},
    {"mon_ch1_sinkSensorResistance", TecFrame::TecParameter::MonSinkSensorResitance, ValueType::Float, Direction::Read,
        "unit", "description", "readCh1Group5", "", "", ""},
    {"mon_ch1_sinkSensorTemp", TecFrame::TecParameter::MonSinkSensorTemperature, ValueType::Float, Direction::Read,
        "unit", "description", "readCh1Group5", "", "", ""},
    {"mon_ch1_objectSensorTemp", TecFrame::TecParameter::MonObjectSensorTemperature, ValueType::Float, Direction::Read,
        "unit", "description", "readCh1Group5", "", "", ""},
    // read-only group 6 for channel 1
    {"mon_ch1_fanRelativeCoolingPower", TecFrame::TecParameter::MonFanRelativeCoolingPower, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group6", "", "", ""},
    {"mon_ch1_fanNominalSpeed", TecFrame::TecParameter::MonFanNominalFanSpeed, ValueType::Float, Direction::Read, "RPM",
        "description", "readCh1Group6", "", "", ""},
    {"mon_ch1_fanActualSpeed", TecFrame::TecParameter::MonFanActualFanSpeed, ValueType::Float, Direction::Read, "RPM",
        "description", "readCh1Group6", "", "", ""},
    {"mon_ch1_fanActualPwmLevel", TecFrame::TecParameter::MonFanActualPwmLevel, ValueType::Float, Direction::Read,
        "unit", "description", "readCh1Group6", "", "", ""},
    // read-only group 7 for channel 1
    {"mon_ch1_objectSensorRawADCValue", TecFrame::TecParameter::MonObjectSensorRawADCValue, ValueType::Int,
        Direction::Read, "unit", "description", "readCh1Group7", "", "", ""},
    {"mon_ch1_sinkSensorRawADCValue", TecFrame::TecParameter::MonSinkSensorRawADCValue, ValueType::Int, Direction::Read,
        "unit", "description", "readCh1Group7", "", "", ""},
    {"mon_ch1_objectSensorType", TecFrame::TecParameter::MonObjectSensorType, ValueType::Int, Direction::Read, "unit",
        "description", "readCh1Group7", "", "", ""},
    // read-only group 8 for channel 1
    {"mon_ch1_objectTempMeasureLowRes", TecFrame::TecParameter::ObjTempMeasureLowestResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group8", "", "", ""},
    {"mon_ch1_objectTempMeasureHighRes", TecFrame::TecParameter::ObjTempMeasureHighestResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group8", "", "", ""},
    // Deliberately short name not to break DOOCS limit later...
    {"mon_ch1_objectTempMeasureAtLowRes", TecFrame::TecParameter::ObjTempMeasureTempAtLowestResistance,
        ValueType::Float, Direction::Read, "unit", "description", "readCh1Group8", "", "", ""},
    // Deliberately short name not to break DOOCS limit later...
    {"mon_ch1_objectTempMeasureAtHighRes", TecFrame::TecParameter::ObjTempMeasureTempAtHighestResistance,
        ValueType::Float, Direction::Read, "unit", "description", "readCh1Group8", "", "", ""},
    // read-only group 9 for channel 1
    {"mon_ch1_sinkTempMeasureLowRes", TecFrame::TecParameter::SinMeasLowestResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group9", "", "", ""},
    {"mon_ch1_sinkTempMeasureHighRes", TecFrame::TecParameter::SinMeasHighestResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group9", "", "", ""},
    {"mon_ch1_sinkTempMeasureAtLowRes", TecFrame::TecParameter::SinMeasTempAtLowestResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group9", "", "", ""},
    {"mon_ch1_sinkTempMeasureAtHighRes", TecFrame::TecParameter::SinMeasTempAtHighestResistance, ValueType::Float,
        Direction::Read, "unit", "description", "readCh1Group9", "", "", ""},

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // READ WRITE parameters – device specific
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    {"ctrl_generalOperatingMode", TecFrame::TecParameter::OpeGeneralOperatingMode, ValueType::Int, Direction::ReadWrite,
        "unit", "description", "readWriteGroup1", "writeGroup1", "writeToFlashGroup1", "setRbGroup1"},
    {"ctrl_deviceAddress", TecFrame::TecParameter::OpeDeviceAddress, ValueType::Int, Direction::ReadWrite, "unit",
        "description", "readWriteGroup1", "writeGroup1", "writeToFlashGroup1", "setRbGroup1"},
    {"ctrl_rs485Channel1BaudRate", TecFrame::TecParameter::OpeRS485CH1BaudRate, ValueType::Int, Direction::ReadWrite,
        "unit", "description", "readWriteGroup1", "writeGroup1", "writeToFlashGroup1", "setRbGroup1"},
    {"ctrl_rs485Channel1ResponseDelay", TecFrame::TecParameter::OpeRS485CH1ResponseDelay, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readWriteGroup1", "writeGroup1", "writeToFlashGroup1",
        "setRbGroup1"},
    {"ctrl_commWatchdogTimeout", TecFrame::TecParameter::OpeComWatchDogTimeout, ValueType::Int, Direction::ReadWrite,
        "unit", "description", "readWriteGroup1", "writeGroup1", "writeToFlashGroup1", "setRbGroup1"},

    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // READ WRITE parameters – Channel specific
    // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    // read-write group 1 for channel 1
    {"ctrl_ch1_targetObjectTemp", TecFrame::TecParameter::TemTargetObjectTemp, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup1", "writeCh1Group1", "writeToFlashCh1Group1", "setRbCh1Group1"},
    {"ctrl_ch1_coarseTempRamp", TecFrame::TecParameter::TemCoarseTempRamp, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup1", "writeCh1Group1", "writeToFlashCh1Group1", "setRbCh1Group1"},
    {"ctrl_ch1_proximityWidth", TecFrame::TecParameter::TemProximityWidth, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup1", "writeCh1Group1", "writeToFlashCh1Group1", "setRbCh1Group1"},
    // read-write group 2 for channel 1
    {"ctrl_ch1_pidKp", TecFrame::TecParameter::TemKp, ValueType::Float, Direction::ReadWrite, "unit", "description",
        "readCh1WriteGroup2", "writeCh1Group2", "writeToFlashCh1Group2", "setRbCh1Group2"},
    {"ctrl_ch1_pidTi", TecFrame::TecParameter::TemTi, ValueType::Float, Direction::ReadWrite, "unit", "description",
        "readCh1WriteGroup2", "writeCh1Group2", "writeToFlashCh1Group2", "setRbCh1Group2"},
    {"ctrl_ch1_pidTd", TecFrame::TecParameter::TemTd, ValueType::Float, Direction::ReadWrite, "unit", "description",
        "readCh1WriteGroup2", "writeCh1Group2", "writeToFlashCh1Group2", "setRbCh1Group2"},
    {"ctrl_ch1_pidDPartDampingPt1", TecFrame::TecParameter::TemDPartDampPT1, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup2", "writeCh1Group2", "writeToFlashCh1Group2", "setRbCh1Group2"},
    // read-write group 3 for channel 1
    {"ctrl_ch1_outStageCurrentLimit", TecFrame::TecParameter::OpeCurrentLimitation, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup3", "writeCh1Group3", "writeToFlashCh1Group3",
        "setRbCh1Group3"},
    {"ctrl_ch1_outStageVoltageLimit", TecFrame::TecParameter::OpeVoltageLimitation, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup3", "writeCh1Group3", "writeToFlashCh1Group3",
        "setRbCh1Group3"},
    {"ctrl_ch1_outStageCurrentErrorThreshold", TecFrame::TecParameter::OpeCurrentErrorThreshold, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup3", "writeCh1Group3", "writeToFlashCh1Group3",
        "setRbCh1Group3"},
    {"ctrl_ch1_outStageVoltageErrorThreshold", TecFrame::TecParameter::OpeVoltageErrorThreshold, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup3", "writeCh1Group3", "writeToFlashCh1Group3",
        "setRbCh1Group3"},
    // read-write group 4 for channel 1
    {"ctrl_ch1_thermalPowerRegulationMode", TecFrame::TecParameter::TemModelizationMode, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4",
        "setRbCh1Group4"},
    {"ctrl_ch1_peltierMaxCurrent", TecFrame::TecParameter::TemPeltierMaxCurrent, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4", "setRbCh1Group4"},
    {"ctrl_ch1_peltierMaxVoltage", TecFrame::TecParameter::TemPeltierMaxVoltage, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4", "setRbCh1Group4"},
    {"ctrl_ch1_peltierCoolingCapacity", TecFrame::TecParameter::TemPeltierCoolingCapacity, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4",
        "setRbCh1Group4"},
    {"ctrl_ch1_peltierDeltaTemp", TecFrame::TecParameter::TemPeltierDeltaTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4",
        "setRbCh1Group4"},
    {"ctrl_ch1_peltierPositiveCurrentIs", TecFrame::TecParameter::TemPeltierPositiveCurrentIs, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4",
        "setRbCh1Group4"},
    {"ctrl_ch1_resistorResistance", TecFrame::TecParameter::TemResistorResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4",
        "setRbCh1Group4"},
    {"ctrl_ch1_resistorMaxCurrent", TecFrame::TecParameter::TemResistorMaxCurrent, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup4", "writeCh1Group4", "writeToFlashCh1Group4",
        "setRbCh1Group4"},
    // read-write group 5 for channel 1
    {"ctrl_ch1_sinkTempSelection", TecFrame::TecParameter::SinSinkTemperatureSelection, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup5", "writeCh1Group5", "writeToFlashCh1Group5",
        "setRbCh1Group5"},
    {"ctrl_ch1_sinkTempFixedValue", TecFrame::TecParameter::SinFixedTemperature, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup5", "writeCh1Group5", "writeToFlashCh1Group5", "setRbCh1Group5"},
    {"ctrl_ch1_sinkUpperAdcLimitError", TecFrame::TecParameter::SinUpperAdcLimitError, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup5", "writeCh1Group5", "writeToFlashCh1Group5",
        "setRbCh1Group5"},
    // read-write group 6 for channel 1
    {"ctrl_ch1_outputStageStaticCurrent", TecFrame::TecParameter::OpeSetStaticCurrent, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup6", "writeCh1Group6", "writeToFlashCh1Group6",
        "setRbCh1Group6"},
    {"ctrl_ch1_outputStageStaticVoltage", TecFrame::TecParameter::OpeSetStaticVoltage, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup6", "writeCh1Group6", "writeToFlashCh1Group6",
        "setRbCh1Group6"},
    // read-write group 7 for channel 1
    {"ctrl_ch1_fanControlEnable", TecFrame::TecParameter::ExpFanControlEnable, ValueType::Int, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},
    {"ctrl_ch1_fanPwmFrequency", TecFrame::TecParameter::ExpPwmFrequency, ValueType::Int, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},
    {"ctrl_ch1_fanActualTempSource", TecFrame::TecParameter::ExpFanActualTempSource, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7",
        "setRbCh1Group7"},
    {"ctrl_ch1_fanTargetTemp", TecFrame::TecParameter::ExpFanTargetTemp, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},
    {"ctrl_ch1_fanTempKp", TecFrame::TecParameter::ExpFanTempKp, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},
    {"ctrl_ch1_fanTempTi", TecFrame::TecParameter::ExpFanTempTi, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},
    {"ctrl_ch1_fanTempTd", TecFrame::TecParameter::ExpFanTempTd, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup7", "writeCh1Group7", "writeToFlashCh1Group7", "setRbCh1Group7"},
    // read-write group 8 for channel 1
    {"ctrl_ch1_fanSpeedAt0Percent", TecFrame::TecParameter::ExpFanSpeedMin, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8", "setRbCh1Group8"},
    {"ctrl_ch1_fanSpeedAt100Percent", TecFrame::TecParameter::ExpFanSpeedMax, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8", "setRbCh1Group8"},
    {"ctrl_ch1_fanSpeedCtrlKp", TecFrame::TecParameter::ExpFanSpeedKp, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8", "setRbCh1Group8"},
    {"ctrl_ch1_fanSpeedCtrlTi", TecFrame::TecParameter::ExpFanSpeedTi, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8", "setRbCh1Group8"},
    {"ctrl_ch1_fanSpeedCtrlTd", TecFrame::TecParameter::ExpFanSpeedTd, ValueType::Float, Direction::ReadWrite, "unit",
        "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8", "setRbCh1Group8"},
    {"ctrl_ch1_fanBypassSpeedCtrl", TecFrame::TecParameter::ExpFanSpeedBypass, ValueType::Int, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8", "setRbCh1Group8"},
    {"ctrl_ch1_fanSpeedCtrlSurveillance", TecFrame::TecParameter::ExpFanSurveillance, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup8", "writeCh1Group8", "writeToFlashCh1Group8",
        "setRbCh1Group8"},
    // read-write group 9 for channel 1
    {"ctrl_ch1_objectMeasureTempOffset", TecFrame::TecParameter::ObjTemperatureOffset, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup9", "writeCh1Group9", "writeToFlashCh1Group9",
        "setRbCh1Group9"},
    {"ctrl_ch1_objectMeasureTempGain", TecFrame::TecParameter::ObjTemperatureGain, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup9", "writeCh1Group9", "writeToFlashCh1Group9",
        "setRbCh1Group9"},
    {"ctrl_ch1_objectLowerErrorThreshold", TecFrame::TecParameter::ObjLowerErrorThreshold, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup9", "writeCh1Group9", "writeToFlashCh1Group9",
        "setRbCh1Group9"},
    {"ctrl_ch1_objectUpperErrorThreshold", TecFrame::TecParameter::ObjUpperErrorThreshold, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup9", "writeCh1Group9", "writeToFlashCh1Group9",
        "setRbCh1Group9"},
    {"ctrl_ch1_objectMaxTempChange", TecFrame::TecParameter::ObjMaxTempChange, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup9", "writeCh1Group9", "writeToFlashCh1Group9", "setRbCh1Group9"},
    // read-write group 10 for channel 1
    {"ctrl_ch1_objectNtcLowerPointTemp", TecFrame::TecParameter::ObjNTCLowerPointTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup10", "writeCh1Group10", "writeToFlashCh1Group10",
        "setRbCh1Group10"},
    {"ctrl_ch1_objectNtcLowerPointResistance", TecFrame::TecParameter::ObjNTCLowerPointResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup10", "writeCh1Group10", "writeToFlashCh1Group10",
        "setRbCh1Group10"},
    {"ctrl_ch1_objectNtcMiddlePointTemp", TecFrame::TecParameter::ObjNTCMiddlePointTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup10", "writeCh1Group10", "writeToFlashCh1Group10",
        "setRbCh1Group10"},
    {"ctrl_ch1_objectNtcMiddlePointResistance", TecFrame::TecParameter::ObjNTCMiddlePointResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup10", "writeCh1Group10", "writeToFlashCh1Group10",
        "setRbCh1Group10"},
    {"ctrl_ch1_objectNtcUpperPointTemp", TecFrame::TecParameter::ObjNTCUpperPointTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup10", "writeCh1Group10", "writeToFlashCh1Group10",
        "setRbCh1Group10"},
    {"ctrl_ch1_objectNtcUpperPointResistance", TecFrame::TecParameter::ObjNTCUpperPointResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup10", "writeCh1Group10", "writeToFlashCh1Group10",
        "setRbCh1Group10"},
    // read-write group 11 for channel 1
    {"ctrl_ch1_objectStabIndicatorTempDeviation", TecFrame::TecParameter::ObjStabilityTemperatureWindow,
        ValueType::Float, Direction::ReadWrite, "unit", "description", "readCh1WriteGroup11", "writeCh1Group11",
        "writeToFlashCh1Group11", "setRbCh1Group11"},
    {"ctrl_ch1_objectStabIndMinTimeInWindow", TecFrame::TecParameter::ObjStabilityMinTimeInWindow, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup11", "writeCh1Group11", "writeToFlashCh1Group11",
        "setRbCh1Group11"},
    {"ctrl_ch1_objectStabIndicatorMaxStabTime", TecFrame::TecParameter::ObjStabilityMaxStabiTime, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup11", "writeCh1Group11", "writeToFlashCh1Group11",
        "setRbCh1Group11"},
    // read-write group 12 for channel 1
    {"ctrl_ch1_sinkMeasureTempOffset", TecFrame::TecParameter::SinTemperatureOffset, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup12", "writeCh1Group12", "writeToFlashCh1Group12",
        "setRbCh1Group12"},
    {"ctrl_ch1_sinkMeasureTempGain", TecFrame::TecParameter::SinTemperatureGain, ValueType::Float, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup12", "writeCh1Group12", "writeToFlashCh1Group12", "setRbCh1Group12"},
    {"ctrl_ch1_sinkTempErrorLowerThreshold", TecFrame::TecParameter::SinLowerErrorThreshold, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup12", "writeCh1Group12", "writeToFlashCh1Group12",
        "setRbCh1Group12"},
    {"ctrl_ch1_sinkTempErrorUpperThreshold", TecFrame::TecParameter::SinUpperErrorThreshold, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup12", "writeCh1Group12", "writeToFlashCh1Group12",
        "setRbCh1Group12"},
    {"ctrl_ch1_sinkTempErrorMaxTempChange", TecFrame::TecParameter::SinMaxTempChange, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup12", "writeCh1Group12", "writeToFlashCh1Group12",
        "setRbCh1Group12"},
    // read-write group 13 for channel 1
    {"ctrl_ch1_sinkNtcLowerPointTemp", TecFrame::TecParameter::SinNTCLowerPointTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup13", "writeCh1Group13", "writeToFlashCh1Group13",
        "setRbCh1Group13"},
    // read-write group 13 for channel 1
    {"ctrl_ch1_sinkNtcLowerPointResistance", TecFrame::TecParameter::SinNTCLowerPointResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup13", "writeCh1Group13", "writeToFlashCh1Group13",
        "setRbCh1Group13"},
    // read-write group 13 for channel 1
    {"ctrl_ch1_sinkNtcMiddlePointTemp", TecFrame::TecParameter::SinNTCMiddlePointTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup13", "writeCh1Group13", "writeToFlashCh1Group13",
        "setRbCh1Group13"},
    // read-write group 13 for channel 1
    {"ctrl_ch1_sinkNtcMiddlePointResistance", TecFrame::TecParameter::SinNTCMiddlePointResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup13", "writeCh1Group13", "writeToFlashCh1Group13",
        "setRbCh1Group13"},
    // read-write group 13 for channel 1
    {"ctrl_ch1_sinkNtcUpperPointTemp", TecFrame::TecParameter::SinNTCUpperPointTemperature, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup13", "writeCh1Group13", "writeToFlashCh1Group13",
        "setRbCh1Group13"},
    // read-write group 13 for channel 1
    {"ctrl_ch1_sinkNtcUpperPointResistance", TecFrame::TecParameter::SinNTCUpperPointResistance, ValueType::Float,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup13", "writeCh1Group13", "writeToFlashCh1Group13",
        "setRbCh1Group13"},
    // read-write group 14 for channel 1
    {"ctrl_ch1_outputStageEnable", TecFrame::TecParameter::OpeOutputStageEnable, ValueType::Int, Direction::ReadWrite,
        "unit", "description", "readCh1WriteGroup14", "writeCh1Group14", "writeToFlashCh1Group14", "setRbCh1Group14"},
    {"ctrl_ch1_outputStageControlInputSelection", TecFrame::TecParameter::OpeOutputStageInputSelection, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup14", "writeCh1Group14", "writeToFlashCh1Group14",
        "setRbCh1Group14"},
    {"ctrl_ch1_sensorTypeSelection", TecFrame::TecParameter::ExpObjMeasSensorTypeSelection, ValueType::Int,
        Direction::ReadWrite, "unit", "description", "readCh1WriteGroup14", "writeCh1Group14", "writeToFlashCh1Group14",
        "setRbCh1Group14"}};

bool checkDescriptorsList() {
  std::cout << "checkDescriptorsList():: Checking descriptors list structure." << std::endl;
  bool error = false;
  std::map<TecFrame::TecParameter, ParameterDescriptor> tecParameterMap;
  std::map<std::string, ParameterDescriptor> pvNameMap;
  for(const auto& descriptor : descriptors) {
    // first check if there is no twice the same TecFrame::TecParameter
    if(tecParameterMap.find(descriptor.value) == tecParameterMap.end())
      tecParameterMap[descriptor.value] = descriptor;
    else {
      error = true;
      std::cout << "checkDescriptorsList():: Descriptor with TecFrame::TecParameter " << descriptor.value
                << " already in the list." << std::endl;
    }
    // check if there is no twice the same pvName
    if(pvNameMap.find(descriptor.pvName) == pvNameMap.end())
      pvNameMap[descriptor.pvName] = descriptor;
    else {
      error = true;
      std::cout << "checkDescriptorsList():: Descriptor with pvName " << descriptor.pvName << " already in the list."
                << std::endl;
    }
  }

  if(error) {
    std::cout << "checkDescriptorsList():: Descriptors list structure is NOT CORRECT!" << std::endl;
  }
  else {
    std::cout << "checkDescriptorsList():: Descriptors list structure is CORRECT!" << std::endl;
  }

  return !error;
}
